// -*- c++ -*-
/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#if !defined __TUNEPARAMS_H
#define __TUNEPARAMS_H

#include <string>

// Abstract base class for the Tuning Parameters
class TuneParams {
protected:
  TuneParams(const std::string &name, int apid, int vpid)
    : name(name), apid(apid), vpid(vpid)
  {
  }

  virtual ~TuneParams();

public:

  std::string getName() const { return name; };

  // Is the TuneParams the same as another
  // apart from the channel name and the PIDs?
  // (ie. same freq, modulation, fec codes etc).
  virtual bool sameAsExceptPIDs(const TuneParams &other) const = 0;

  // Given the frontendFD, tune the params in.
  virtual bool tune(int frontendFD) const = 0;

  // Get the PIDS
  int getAudioPID() const { return apid; };
  int getVideoPID() const { return vpid; };

  int getNumPIDs() const { return (apid != 0) + (vpid != 0); };

protected:
  std::string name;
  int apid, vpid;
};

class DVBTTuneParams : public TuneParams {
private:
  DVBTTuneParams(const std::string &name, int apid, int vpid);
  virtual ~DVBTTuneParams();
public:

  // Parses a line from ~/.tzap/channels.conf
  // Throws an execption if not well formed.
  static DVBTTuneParams *createFromTZAP(const std::string &line);

  virtual bool sameAsExceptPIDs(const TuneParams &other) const;
  virtual bool tune(int frontendFD) const;

private:
  /* Frequency etc */
  unsigned long freq;
  int qam, inv, fec1, fec2, hier, bandw, mode, guard;
};

class DVBSTuneParams : public TuneParams {
private:
  DVBSTuneParams(const std::string &name, int apid, int vpid);
  virtual ~DVBSTuneParams();
public:

  // Parses a line from ~/.szap/channels.conf (or whatever is set to be a config file)
  // Throws an execption if not well formed.
  static DVBSTuneParams *createFromSZAP(const std::string &line);

  virtual bool sameAsExceptPIDs(const TuneParams &other) const;
  virtual bool tune(int frontendFD) const;

private:
  /* Frequency etc */
  unsigned long freq;
  int sr, sat_no, pol;
};

class DVBCTuneParams : public TuneParams {
private:
  DVBCTuneParams(const std::string &name, int apid, int vpid);
  virtual ~DVBCTuneParams();
public:

  // Parses a line from ~/.czap/channels.conf
  // Throws an execption if not well formed.
  static DVBCTuneParams *createFromCZAP(const std::string &line);

  virtual bool sameAsExceptPIDs(const TuneParams &other) const;
  virtual bool tune(int frontendFD) const;

private:
  /* Frequency etc */
  unsigned long freq;
  int inv, sr, fec, qam; 
};

#endif // __TUNEPARAMS_H
